import os
import traceback

from django.apps import AppConfig

from .classes.app_mixins.auto_migration_mixin import AutoMigrationMixin
from .classes.app_mixins.auto_urlpatterns_mixin import AutoUrlPatternsMixin


class ValarConfig(AutoMigrationMixin, AutoUrlPatternsMixin, AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = __package__

    def ready(self):
        if os.environ.get('RUN_MAIN') == 'true':
            try:
                from .dao.frame import MetaFrame
                # getattr(super(), 'set_url', None)()
                # getattr(super(), 'auto_migrate', None)()
                MetaFrame()
            except Exception as e:
                traceback.print_exc()
                print('ERROR', e)



meta_field_value_defaults = {
    'valar.Meta': {
        "__init__": {
            "db": {
                "tool": "set",
                "format": {
                    "set": {
                        'orm': 'SQL',
                        'mon': 'MongoDB',
                        'center': '剧中对齐',
                    }
                }
            },
        }
    },
    'valar.MetaView': {
        "__init__": {
            "meta_id": {
                'allow_edit': False,
            },
            "code": {
                'allow_edit': False,
            },
            "name": {
                'span': 24
            }
        },
        "core": {
            "name": {
                'span': 16
            },
            "enable": {
                "span": 8
            }
        }
    },
    'valar.MetaFieldDomain': {
        "__init__": {
            "default_id": {
                "tool": "tree"
            },
            "search_id": {
                "tool": "tree"
            },
            "tools": {
                "tool": "tree",
                "refer": {
                    "display": "code"
                }
            },
            "align": {
                "tool": "set",
                "format": {
                    "set": {
                        'left': '左对齐',
                        'right': '右对齐',
                        'center': '剧中对齐',
                    }
                }
            }
        }
    },

    'valar.MetaField': {
        "rest": {
            "name": {
                'hide_on_form': False
            },
        },
        "__init__": {
            "column_width": {
                'unit': 'px'
            },
            "name": {
                'span': 24,
                'hide_on_form': True
            },
            "fixed": {
                "tool": "set",
                "format": {
                    "set": {
                        'left': '左侧固定',
                        'right': '右侧固定',
                    }
                }
            },
            "align": {
                "tool": "set",
                "format": {
                    "set": {
                        'left': '左对齐',
                        'right': '右对齐',
                        'center': '剧中对齐',
                    }
                }
            },
            "prop": {
                'allow_edit': False,
                'column_width': 120
            },
            "domain": {
                'allow_edit': False,
                'column_width': 120,
            },
            "tool": {
                'column_width': 100,
                'span': 24,
                'tool': 'tree',
                'refer': {
                    'entity': 'valar.MetaFieldTool',
                    'includes': {'metafielddomain__name': '${domain}'},
                    'value': 'code', 'display': 'code', "isTree": True
                }

            },
            "span": {
                'column_width': 100,
                "format": {"min": 0, "max": 24, "step": 1, "precision": 0, "step_strictly": True}
            },
            "refer": {
                'allow_edit': False,
                'column_width': 80
            },
            "format": {
                'allow_edit': False,
                'column_width': 80
            },
            'header_color': {
                'tool': 'color',
            },
            'cell_color': {
                'tool': 'color',
            }
        }
    },
    'valar.Indicator': {
        "__init__": {
            "frequency": {
                'column_width': 80,
            },
            "unit": {
                'column_width': 80,
            },
            "scope": {
                'column_width': 80,
            },
            "file": {
                'column_width': 90,
            }
        },
    }
}



from ..models.core import VTree, VModel
from django.db import models


class IndicatorSet(VTree):
    name = models.CharField(max_length=100, null=True)

    class Meta:
        verbose_name = '指标集'


class Indicator(VModel):
    set = models.ForeignKey(IndicatorSet, null=True, on_delete=models.CASCADE, verbose_name='指标集')
    domain = models.CharField(max_length=100, verbose_name='domain', null=True)
    category = models.CharField(max_length=100, verbose_name='category', null=True)
    name = models.CharField(max_length=100, verbose_name='名称', null=True)
    frequency = models.CharField(max_length=100, verbose_name='频度', null=True)
    scope = models.CharField(max_length=100, verbose_name='尺度', null=True)
    unit = models.CharField(max_length=100, verbose_name='单位', null=True)
    object_id = models.CharField(max_length=100, verbose_name='ObjectId', null=True)
    file = models.FileField(null=True, verbose_name='数据文件')

    class Meta:
        verbose_name = '指标项'


class Location(VModel):
    scope = models.CharField(max_length=100, null=True)
    code = models.CharField(max_length=100, null=True)
    name = models.CharField(max_length=100, null=True)
    name_cn = models.CharField(max_length=100, null=True)
    name_en = models.CharField(max_length=100, null=True)

    class Meta:
        verbose_name = '指标集'



meta_view_default_values = {
    'valar.Meta': {
        '__init__': {
            'allow_insert': False,
            'allow_upload': False,
            'lock': True
        }
    },
    'valar.MetaView': {
        '__init__': {
            'allow_insert': False,
            'allow_upload': False,
            'lock': True
        },
    },
    'valar.MetaField': {
        '__init__': {
            'allow_insert': False,
            'allow_upload': False,
            'lock': True
        },
        'tool': {
            'allow_edit_on_form': False,
        }
    }
}



# Generated by Django 4.2.24 on 2025-09-22 11:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('valar', '0002_location'),
    ]

    operations = [
        migrations.AddField(
            model_name='location',
            name='code',
            field=models.CharField(max_length=100, null=True),
        ),
        migrations.AddField(
            model_name='location',
            name='name_cn',
            field=models.CharField(max_length=100, null=True),
        ),
        migrations.AddField(
            model_name='location',
            name='name_en',
            field=models.CharField(max_length=100, null=True),
        ),
    ]



# Generated by Django 4.2.24 on 2025-09-22 11:52

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('valar', '0003_location_code_location_name_cn_location_name_en'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='location',
            name='icon',
        ),
        migrations.RemoveField(
            model_name='location',
            name='isLeaf',
        ),
        migrations.RemoveField(
            model_name='location',
            name='pid',
        ),
    ]



import base64
import json

from django.http import HttpRequest
from django.utils.deprecation import MiddlewareMixin

from ..auth.Authentication import valid_request_token, ValaAuthError
from ..classes.valar_response import ValarResponse


class ValarMiddleware(MiddlewareMixin):
    @staticmethod
    def process_response(request: HttpRequest, response: ValarResponse):
        try:
            payload = valid_request_token(request)
            response["token"] = payload['token'] or '~~~'
            """此处可以加入数据级别的权限验证"""
        except ValaAuthError as e:
            if request.headers.get("Auth"):
                return ValarResponse(False, '无效权限', 'error', status=403)

        if type(response) == ValarResponse:
            valar_message, valar_code = response.valar_message, response.valar_code
            data = {
                'payload': json.loads(response.content),
                'message': valar_message,
                'code': valar_code,
            }
            response.content = json.dumps(data, ensure_ascii=False).encode("utf-8")
            response["Content-Type"] = "application/json; charset=utf-8"

        return response



import copy

from ..classes.singleton_meta import SingletonMeta


class MetaFrame(metaclass=SingletonMeta):
    def __init__(self):
        from .orm_dao import OrmDao
        from .meta import ValarMeta
        from ..models.core import VTree
        from .engine import ValarEngine
        tool_dao = OrmDao('valar.MetaFieldTool')
        domain_dao = OrmDao('valar.MetaFieldDomain')
        tool_dao.delete()
        domain_dao.delete()
        mapping = {}
        for item in meta_field_tool:
            item = copy.deepcopy(item)
            _id, code = item['id'], item['code']
            item.update({"saved": True})
            if item['isLeaf']:
                mapping[code] = _id
            tool_dao.save_one(item)
        for row in meta_field_domain:
            row = copy.deepcopy(row)
            default_id, tools = row['default_id'], row['tools']
            _row = copy.deepcopy(row)
            _row.update({
                'default_id': mapping[default_id],
                'tools': [mapping[tool] for tool in tools],
                "saved": True
            })
            domain_dao.save_one(_row)

        meta_dao = OrmDao('valar.Meta')
        meta_dao.delete([{"includes": {'db__exact': 'orm', 'entity__startswith': 'valar.'}}])
        engine = ValarEngine()
        for entity, model in engine.orm_engine.items():
            tree = issubclass(model, VTree)
            ValarMeta('orm', entity, 'default')
        cols = engine.mongo_engine.list_collections()
        for col in cols:
            entity = col['name']
            ValarMeta('mon', entity, 'default')


meta_field_tool = [
    {'id': 2, 'sort': 32, 'pid': 7, 'isLeaf': True, 'name': '输入框', 'code': 'text', 'align': 'left'},
    {'id': 3, 'sort': 17, 'pid': 0, 'isLeaf': False, 'name': 'SPEC', 'code': '特殊工具集'},
    {'id': 5, 'sort': 22, 'pid': 0, 'isLeaf': False, 'name': 'DATE', 'code': '日期时间工具集'},
    {'id': 6, 'sort': 21, 'pid': 8, 'isLeaf': True, 'name': '数字输入', 'code': 'number', 'align': 'right'},
    {'id': 7, 'sort': 36, 'pid': 0, 'isLeaf': False, 'name': 'TEXT', 'code': '文本工具集'},
    {'id': 8, 'sort': 26, 'pid': 0, 'isLeaf': False, 'name': 'NUMB', 'code': '数字工具集'},
    {'id': 9, 'sort': 10, 'pid': 0, 'isLeaf': False, 'name': 'FILE', 'code': '文件工具集'},
    {'id': 10, 'sort': 27, 'pid': 0, 'isLeaf': False, 'name': 'BOOL', 'code': '逻辑工具集'},
    {'id': 11, 'sort': 31, 'pid': 0, 'isLeaf': False, 'name': 'LIST', 'code': '列表工具集'},
    {'id': 12, 'sort': 8, 'pid': 3, 'isLeaf': True, 'name': '对象', 'code': 'object', 'align': 'center'},
    {'id': 13, 'sort': 5, 'pid': 9, 'isLeaf': True, 'name': '图片上传', 'code': 'image', 'align': 'center'},
    {'id': 14, 'sort': 2, 'pid': 9, 'isLeaf': True, 'name': '文件上传', 'code': 'file', 'align': 'center'},
    {'id': 15, 'sort': 13, 'pid': 7, 'isLeaf': True, 'name': '富文本', 'code': 'rich', 'align': 'center'},
    {'id': 17, 'sort': 11, 'pid': 10, 'isLeaf': True, 'name': '开关', 'code': 'switch', 'align': 'center'},
    {'id': 19, 'sort': 9, 'pid': 7, 'isLeaf': True, 'name': '颜色选择', 'code': 'color', 'align': 'center'},
    {'id': 20, 'sort': 14, 'pid': 11, 'isLeaf': True, 'name': '穿梭框', 'code': 'transfer', 'align': 'left'},
    {'id': 21, 'sort': 16, 'pid': 7, 'isLeaf': True, 'name': '自动填充', 'code': 'auto', 'align': 'left'},
    {'id': 22, 'sort': 35, 'pid': 5, 'isLeaf': True, 'name': '日期选择', 'code': 'date', 'align': 'left'},
    {'id': 23, 'sort': 12, 'pid': 10, 'isLeaf': True, 'name': '逻辑选择', 'code': 'boolean', 'align': 'center'},
    {'id': 24, 'sort': 24, 'pid': 11, 'isLeaf': True, 'name': '列表选择', 'code': 'select', 'align': 'left'},
    {'id': 25, 'sort': 15, 'pid': 11, 'isLeaf': True, 'name': '树形选择', 'code': 'tree', 'align': 'left'},
    {'id': 26, 'sort': 23, 'pid': 11, 'isLeaf': True, 'name': '及联选择', 'code': 'cascade', 'align': 'left'},
    {'id': 28, 'sort': 25, 'pid': 7, 'isLeaf': True, 'name': '图标', 'code': 'icon', 'align': 'center'},
    {'id': 32, 'sort': 30, 'pid': 7, 'isLeaf': True, 'name': '文本框', 'code': 'textarea', 'align': 'left'},
    {'id': 33, 'sort': 18, 'pid': 36, 'isLeaf': True, 'name': '时间区间', 'code': 'timerange', 'align': 'left'},
    {'id': 35, 'sort': 33, 'pid': 5, 'isLeaf': True, 'name': '时间选择', 'code': 'time', 'align': 'left'},
    {'id': 36, 'sort': 20, 'pid': 0, 'isLeaf': False, 'name': 'RANGE', 'code': '区间工具集'},
    {'id': 37, 'sort': 38, 'pid': 36, 'isLeaf': True, 'name': '日期区间', 'code': 'daterange', 'align': 'left'},
    {'id': 39, 'sort': 3, 'pid': 36, 'isLeaf': True, 'name': '多日期', 'code': 'dates', 'align': 'left'},
    {'id': 54, 'sort': 54, 'pid': 7, 'isLeaf': True, 'name': '集合', 'code': 'set', 'align': 'left'},
    {'id': 31, 'sort': 6, 'pid': 0, 'isLeaf': True, 'name': '无', 'code': 'none', 'align': 'left'},
]

meta_field_tool_mapping = {
    row['code']: row['align']
    for row in meta_field_tool
    if row['isLeaf']
}
meta_field_domain = [
    {
        'name': 'CharField',
        'default_id': 'text',
        'tools': [
            'text', 'number', 'color', 'auto', 'date', 'time', 'select', 'tree', 'cascade', 'icon',
            'textarea', 'timerange', 'daterange', 'dates', 'set'
        ]
    },
    {
        'name': 'TextField',
        'default_id': 'textarea',
        'tools': ['text', 'textarea', 'rich']
    },
    {
        'name': 'BooleanField',
        'default_id': 'switch',
        'tools': ['switch', 'boolean']
    },
    {
        'name': 'IntegerField',
        'default_id': 'number',
        'tools': ['number']
    },
    {
        'name': 'FloatField',
        'default_id': 'number',
        'tools': ['number']
    },
    {
        'name': 'ForeignKey',
        'default_id': 'select',
        'tools': ['select', 'tree', 'cascade']
    },
    {
        'name': 'ManyToOneRel',
        'default_id': 'select',
        'tools': ['transfer', 'select', 'tree', 'cascade']
    },
    {
        'name': 'ManyToManyField',
        'default_id': 'select',
        'tools': ['transfer', 'select', 'tree', 'cascade']
    },
    {
        'name': 'ManyToManyRel',
        'default_id': 'select',
        'tools': ['transfer', 'select', 'tree', 'cascade']
    },
    {
        'name': 'OneToOneRel',
        'default_id': 'select',
        'tools': ['select']
    },
    {
        'name': 'OneToOneField',
        'default_id': 'select',
        'tools': ['select']
    },
    {
        'name': 'DateField',
        'default_id': 'date',
        'tools': ['date']
    },
    {
        'name': 'TimeField',
        'default_id': 'time',
        'tools': ['time']
    },
    {
        'name': 'DateTimeField',
        'default_id': 'date',
        'tools': ['date']
    },
    {
        'name': 'JSONField',
        'default_id': 'object',
        'tools': ['object']
    },
    {
        'name': 'FileField',
        'default_id': 'file',
        'tools': ['image', 'file']
    },
    {
        'name': 'BigAutoField',
        'default_id': 'none',
        'tools': ['none']
    },
    {
        'name': 'UUIDField',
        'default_id': 'none',
        'tools': ['none']
    },
    {
        'name': 'Custom',
        'default_id': 'none',
        'tools': ['none']
    },
]
meta_field_domain_mapping = {
    row['name']: row['default_id']
    for row in meta_field_domain
}



import jwt
from django.conf import settings

from ..classes.valar_response import ValarResponse


class ValaAuthError(Exception):
    def __init__(self, message, code):
        self.code = code
        self.message = message


def auth_required(view_func):
    def wrapper(request, *args, **kwargs):
        try:
            payload = valid_request_token(request)
            request.user_id = payload["user_id"]
            return view_func(request, *args, **kwargs)
        except ValaAuthError as e:
            return ValarResponse(False, e.message, e.code, status=401)

    return wrapper


def get_token_from_request(request):
    auth_header = request.headers.get("Authorization")
    if auth_header and auth_header.startswith("Bearer "):
        return auth_header.split(" ")[1]
    return None


status = {
    0: (None, None, None),
    1: ('请登录系统', 'info', 401),
    2: ('状态已过期，请重新登录', 'warning', 401),
    3: ('错误状态，请重新登录', 'error', 401),
}


def valid_request_token(request):
    token = get_token_from_request(request)
    if not token: raise ValaAuthError('请登录系统', 'info')
    try:
        payload = jwt.decode(token, settings.SECRET_KEY, algorithms=["HS256"])
        payload['token'] = token
        return payload
    except jwt.ExpiredSignatureError:
        raise ValaAuthError('状态已过期，请重新登录', 'warning')
    except jwt.InvalidTokenError:
        raise ValaAuthError('错误状态，请重新登录', 'error')



import warnings

from typing_extensions import deprecated

from ..dao.abstract import AbstractDao
from ..dao.mon_dao import MonDao
from ..dao.orm_dao import OrmDao


class Dao(AbstractDao):

    def __init__(self, entity, db='orm'):
        self.dao = OrmDao(entity) if db == 'orm' else MonDao(entity)
        self.db = db
        self.entity = entity
        self.name = self.dao.name
        self.is_tree = self.dao.is_tree
        self.fields = self.dao.fields
        self.manager = self.dao.manager

    def insert_one(self, item):
        return self.dao.insert_one(item)

    def update_one(self, item):
        return self.dao.update_one(item)

    def save_many(self, array: list):
        return self.dao.save_many(array)

    def save_one(self, item):
        return self.dao.save_one(item)

    def delete_one(self, _id):
        return self.dao.delete_one(_id)

    def find_one(self, _id):
        return self.dao.find_one(_id)

    def find(self, conditions=None, orders=None, size=0, page=1):
        return self.dao.find(conditions, orders, size, page)

    def values(self, conditions, props):
        return self.dao.values(conditions, props)

    def update(self, template, conditions):
        return self.dao.update(template, conditions)

    def delete(self, conditions=None) -> list:
        return self.dao.delete(conditions)

    def serialize(self, o, code=None):
        return self.dao.serialize(o, code)

    def tree(self, root, conditions=None):
        return self.dao.tree(root, conditions)



import traceback

from ..channels.counter import Counter
from ..channels.sender import ValarChannelSender
from ..dao import Dao
from ..dao.engine import ValarEngine

import time


def valar_test_handler(sender: ValarChannelSender):
    data = sender.data
    length = data.get('length', 100)
    counter = Counter(length)
    for i in range(length):
        time.sleep(0.1)
        tick = counter.tick()
        tick.update({'name': 'test1'})
        sender.load(tick)


def batch_handler(sender: ValarChannelSender):
    data = sender.data
    entity, db, method = data.get("entity"), data.get("db"), data.get("method")
    dao = Dao(entity, db)
    body = {"method": method}

    try:
        if method == 'save_many':
            array = data.get("array", [])
            counter = Counter(array)
            keys = []
            for item in array:
                item.update({'saved': True})
                bean = dao.save_one(item)
                keys.append(bean.id)
                payload = counter.tick()
                sender.load(payload)
            body.update({'keys': keys})
            sender.done(body)
        elif method == 'delete_many':
            conditions = data.get("conditions", [])
            paths = dao.delete(conditions)
            counter = Counter(len(paths))
            minio = ValarEngine().get_minio_bucket(entity)
            for path in paths:
                minio.remove(path)
                payload = counter.tick()
                sender.load(payload)
            sender.done(body)
    except Exception as e:
        traceback.print_exc()
        sender.error(str(e))



import json
import time
from datetime import datetime

from asgiref.sync import async_to_sync
from channels.layers import get_channel_layer
from django.http import HttpRequest
import threading
from .consumer import VALAR_CHANNEL_GROUP
from ..auth.Authentication import valid_request_token


class Channel:

    def __init__(self, request: HttpRequest):
        body = json.loads(request.body)
        self.handler = body.get('handler')
        self.url = body.get('url')
        self.auth = body.get('auth')
        self.broadcast = body.get('broadcast')
        self.data = body.get('data')
        self.resolve = body.get('resolve')

    def to_dict(self, status, payload):
        data = {
            'handler': self.handler,
            'url': self.url,
            'auth': self.auth,
            'broadcast': self.broadcast,
            'resolve': self.resolve,
            'timestamp': datetime.now().timestamp()
        }
        if status:
            data.update({'status': status})
        if payload:
            data.update({'payload': payload})
        return data


class Sender:

    def __init__(self, request: HttpRequest):
        self.client = request.headers.get('CLIENT')
        # self.uid = request.session.get('UID')
        self.group_send = async_to_sync(get_channel_layer().group_send)


class ValarChannelSender(Sender):

    def __init__(self, request: HttpRequest, interval=1):
        super().__init__(request)
        self.__channel__ = Channel(request)
        self.data = self.__channel__.data
        self.__payload__ = None
        self.__loading__ = False
        self.__thread__ = None
        self.__lock__ = threading.Lock()
        self.__interval__ = interval
        if self.__channel__.auth:
            payload = valid_request_token(request)
            self.user_id = payload["user_id"]

    def _run(self):
        while self.__loading__:
            self.__emit__()
            time.sleep(self.__interval__)

    def start(self):
        if self.__loading__:
            return  # 避免重复启动
        self.__payload__ = None
        self.__loading__ = True
        self.__emit__('start')
        self.__thread__ = threading.Thread(target=self._run, daemon=True)
        self.__thread__.start()

    def stop(self):
        self.__payload__ = None
        self.__loading__ = False
        self.__emit__('stop')
        if self.__thread__:
            self.__thread__.join()
            self.__thread__ = None

    def load(self, payload):
        with self.__lock__:
            self.__payload__ = payload

    def done(self, payload):
        self.__emit__('done', payload)

    def error(self, payload):
        self.__emit__('error', payload)

    def __emit__(self, status='proceed', data=None):
        scope = 'broadcast' if self.__channel__.broadcast else 'client'
        body = {
            'type': f'{scope}.emit',
            'data': self.__channel__.to_dict(status, data or self.__payload__),
            'clients': [self.client],
            'users': []
        }
        self.group_send(VALAR_CHANNEL_GROUP, body)


class ValarSocketSender(Sender):
    def __init__(self, request: HttpRequest):
        super().__init__(request)
        if self.uid:
            body = {'type': 'broadcast.emit', 'uid': self.uid, 'client': self.client}
            self.group_send(VALAR_CHANNEL_GROUP, body)

    def to_users(self, payload, users: list):
        body = {'type': 'user.emit', 'data': payload, 'users': users}
        self.group_send(VALAR_CHANNEL_GROUP, body)

    def to_clients(self, payload, clients: list):
        body = {'type': 'client.emit', 'data': payload, 'clients': clients}
        self.group_send(VALAR_CHANNEL_GROUP, body)

    def broadcast(self, payload):
        body = {'type': 'broadcast.emit', 'data': payload}
        self.group_send(VALAR_CHANNEL_GROUP, body)



# Generated by Django 4.2.24 on 2025-09-22 11:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('valar', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Location',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort', models.BigIntegerField(null=True, verbose_name='序号')),
                ('create_time', models.DateTimeField(auto_now_add=True, null=True, verbose_name='创建时间')),
                ('modify_time', models.DateTimeField(auto_now=True, null=True, verbose_name='修改时间')),
                ('disabled', models.BooleanField(default=False, verbose_name='禁用')),
                ('saved', models.BooleanField(default=False, verbose_name='已保存')),
                ('pid', models.IntegerField(default=0, verbose_name='父节点')),
                ('isLeaf', models.BooleanField(default=False, verbose_name='叶子节点')),
                ('icon', models.CharField(max_length=255, null=True, verbose_name='图标')),
                ('scope', models.CharField(max_length=100, null=True)),
                ('name', models.CharField(max_length=100, null=True)),
            ],
            options={
                'verbose_name': '指标集',
            },
        ),
    ]



from django.db import models
from .core import VModel, VTree


class Valar(VModel):
    text_field = models.TextField(null=True, verbose_name='Text Field')
    boolean_field = models.BooleanField(null=True, verbose_name='Boolean Field')
    integer_field = models.IntegerField(null=True, verbose_name='Integer Field')
    float_field = models.FloatField(null=True, verbose_name='Float Field')
    date_field = models.DateField(null=True, verbose_name='Date Field')
    datetime_field = models.DateTimeField(null=True, verbose_name='Datetime Field')
    time_field = models.TimeField(null=True, verbose_name='Time Field')
    json_field = models.JSONField(null=True, verbose_name='Json Field')
    file = models.FileField(null=True, verbose_name='File Field')


class Vmo(VModel):
    valar = models.ForeignKey(to=Valar, null=True, on_delete=models.CASCADE, verbose_name='Valar')
    name = models.CharField(max_length=100, null=True, verbose_name='Name')


class Voo(VModel):
    valar = models.OneToOneField(to=Valar, null=True, on_delete=models.CASCADE, verbose_name='Valar')
    name = models.CharField(max_length=100, null=True, verbose_name='Name')


class Vmm(VTree):
    valars = models.ManyToManyField(to=Valar, verbose_name='valars')
    name = models.CharField(max_length=100, null=True, verbose_name='name')



import json
import secrets
from django.contrib.auth.hashers import check_password, make_password

from ..auth.Authentication import auth_required
from ..classes.valar_response import ValarResponse

from ..dao.engine import ValarEngine
from ..models.auth import Account, AbstractUser


def send_password(request):
    body = json.loads(request.body)
    account_id = body.get('account_id')
    account = Account.objects.filter(id=account_id).first()
    return __send__(account)


def retrieve_password(request):
    includes = json.loads(request.body)
    account = Account.objects.filter(**includes).first()
    account.token = account.token or secrets.token_hex(16)
    account.save()
    if account:
        return __send__(account)
    else:
        return ValarResponse(False, '该邮箱未在系统中注册', code='error')


@auth_required
def change_password(request):
    body = json.loads(request.body)
    old_password = body.get('old_password')
    new_password = body.get('new_password')
    account_id = request.user_id
    account = Account.objects.get(id=account_id)
    if not account:
        return ValarResponse(False, '账户不存在', code='error')
    if not account.is_auth(old_password):
        return ValarResponse(False, '当前密码错误', code='error')
    if old_password == new_password:
        return ValarResponse(False, '新密码不能与当前密码一致', code='error')
    account.password = make_password(new_password)
    account.token = None
    account.save()
    return ValarResponse(True)


def __send__(account):
    if not account:
        return ValarResponse(False, '账户不存在', code='error')
    token = account.token
    if not token:
        return ValarResponse(False, '未生成临时密码', code='error')
    email = account.email
    if not email:
        return ValarResponse(False, '该账户未登记邮箱信息', code='error')
    content = f'Your temporary password is {token}, please change it as soon as possible.'
    ValarEngine().send_email('Retrieve Password', content, email)
    return ValarResponse(True, '临时密码发送成功', 'success')



import copy

from deepmerge import always_merger
from django.conf import settings

from ..dao.defaults.field_keys_default import meta_field_key_defaults
from ..dao.defaults.field_values_default import meta_field_value_defaults
from ..dao.defaults.view_defaults import meta_view_default_values
from ..dao.abstract import AbstractDao
from ..dao.frame import meta_field_tool_mapping
from ..dao.mon_dao import MonDao
from ..dao.orm_dao import OrmDao
from ..dao.orm_field import column_width
from ..models.frame import MetaFieldDomain
from ..models.meta import Meta, MetaView


class ValarMeta:
    def __init__(self, db, entity, code):
        self.entity = entity
        self.db = db
        self.dao: AbstractDao = MonDao(entity) if db == 'mon' else OrmDao(entity)
        self.code = code or 'default'

        meta_dao = OrmDao('valar.Meta')
        view_dao = OrmDao('valar.MetaView')
        field_dao = OrmDao('valar.MetaField')

        meta_item = {"entity": entity, 'db': db}
        meta: Meta = meta_dao.search(meta_item).first()
        if meta is None:
            meta_item.update({"name": self.dao.name, 'saved': True, 'tree': self.dao.is_tree})
            meta = meta_dao.save_one(meta_item)
        self.meta = meta

        # load_view
        view_item = {"code": self.code, "meta_id": meta.id}
        view: MetaView = view_dao.search(view_item).first()
        if view is None:
            self.__initial_view__(view_item)
            view = view_dao.save_one(view_item)
        self.view = view

        # load_fields
        if view.metafield_set.count() == 0:
            _fields = self.__initial_fields__()
            for _field in _fields:
                if (_field['prop'] not in ['id', 'sort', 'create_time', 'modify_time', 'saved', 'disabled']
                        and _field['domain'] not in ['ManyToManyRel', 'OneToOneRel', 'ManyToOneRel']):
                    _field.update({'view_id': view.id, "saved": True})
                    field_dao.save_one(_field)

    def meta_view(self):
        _view = self.view.json()
        meta = self.meta.json()
        name, entity = meta['name'], meta['entity']
        fields = self.view.metafield_set.all().order_by('-sort')
        _fields = {}
        _minio = settings.MINIO_ROOT if hasattr(settings, 'MINIO_ROOT') else '/minio'
        for field in fields:
            _field = field.json(entity=entity, code=self.code, db='orm', minio=_minio)
            align, tool, width, domain = _field['align'], _field['tool'], _field['column_width'], _field['domain']
            _field['align'] = align if align else meta_field_tool_mapping.get(tool, 'left')
            _field['column_width'] = width if width else column_width(domain, tool)

            _fields[field.prop] = _field

        _minio = settings.MINIO_ROOT if hasattr(settings, 'MINIO_ROOT') else '/minio'
        _view.update({
            '$minio': _minio,
            '$db': 'orm',
            '$entity': entity,
            '$code': self.code,
            '$meta_name': name,
            '$isTree': self.dao.is_tree,
            '$fields': _fields,
            '$modes': self.dao.full_props()
        })
        return _view

    """ 默认的view设置 """

    def __initial_view__(self, view_item: dict):
        default_view = meta_view_default_values.get(self.entity, {})
        default_values = copy.deepcopy(default_view.get('__init__', {}))
        code_values = copy.deepcopy(default_view.get(self.code, {}))
        view_item.update({
            "name": self.code.upper(),
            "saved": True,
            "property": {
                "value": "id",
                "label": "name",
                "display": "id",
                "sort": "sort",
                "disabled": "disabled",
                "isLeaf": "isLeaf",
                "icon": "icon",
                "children": "children",
                "pid": "pid",
            }
        })
        values = always_merger.merge(default_values, code_values)
        view_item.update(values)

    def __initial_fields__(self):
        default_keys = meta_field_key_defaults.get(self.entity, {})
        method, array = default_keys.get(self.code, ('omit', []))

        def fun(prop):
            return prop not in array if method == 'omit' else prop in array

        props = [prop for prop in self.dao.props() if fun(prop)]
        if method == 'pick':
            props = [prop for prop in array if prop in props]

        default_values = copy.deepcopy(meta_field_value_defaults.get(self.entity, {}))
        init_values = default_values.get('__init__', {})
        code_values = default_values.get(self.code, {})
        default_fields = always_merger.merge(init_values, code_values)

        fields = []
        for prop in props:
            field = self.dao.get_meta_field(prop)
            field_json = field.to_dict()
            default_field = default_fields.get(prop, {})
            always_merger.merge(field_json, default_field)
            fields.append(field_json)
        fields.reverse()
        return fields
